# pylint: disable=too-many-lines
# pylint: disable=line-too-long
"""The slist commands module.

These commands are used in the following models:
AWG5200, AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SLISt:NAME? <sequence_list_index>
    - SLISt:SEQuence:AMPLitude <sequence_name>,<amplitude>? <sequence_name>
    - SLISt:SEQuence:DELete {<sequence_name>|ALL}
    - SLISt:SEQuence:EVENt:JTIMing <sequence_name>, {END|IMMediate}? <sequence_name>
    - SLISt:SEQuence:EVENt:PJUMp:DEFine <sequence_name>, <pattern>, <jump_step>? <sequence_name>, <pattern>
    - SLISt:SEQuence:EVENt:PJUMp:ENABle <sequence_name>, {0|1|OFF|ON}? <sequence_name>
    - SLISt:SEQuence:EVENt:PJUMp:SIZE?
    - SLISt:SEQuence:FREQuency <seq_name>,<frequency>? <seq_name>
    - SLISt:SEQuence:LENGth? <sequence_name>
    - SLISt:SEQuence:NEW <sequence_name>,<number_of_steps>[,<number_of_tracks>]
    - SLISt:SEQuence:OFFSet <sequence_name>,<offset>? <sequence_name>
    - SLISt:SEQuence:RFLag <sequence_name>, {0|1|OFF|ON}? <sequence_name>
    - SLISt:SEQuence:SRATe <sequence_name>,<sample_rate>? <sequence_name>
    - SLISt:SEQuence:STEP:ADD <sequence_name>,<location>[,<steps to add>]
    - SLISt:SEQuence:STEP:MAX?
    - SLISt:SEQuence:STEP:RCOunt:MAX?
    - SLISt:SEQuence:STEP[n]:EJINput <sequence_name>,{ATRigger|BTRigger|OFF|ITRigger}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:EJUMp <sequence_name>, {<NR1>|NEXT|FIRSt|LAST|END}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:GOTO <sequence_name>, {<NR1>|LAST|FIRSt|NEXT|END}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:RCOunt <sequence_name>, {ONCE|INFinite|<NR1>}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TASSet:SEQuence <sequence_name>, <subsequence_name>
    - SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform <sequence_name>, <waveform_name>
    - SLISt:SEQuence:STEP[n]:TASSet[m]? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>
    - SLISt:SEQuence:STEP[n]:WINPut <sequence_name>, {ATRigger|BTRigger|ITRigger|OFF}? <sequence_name>
    - SLISt:SEQuence:TRACk:MAX?
    - SLISt:SEQuence:TRACk? <sequence_name>
    - SLISt:SEQuence:TSTamp? <sequence_name>
    - SLISt:SEQuence:WMUSage? <sequence_name>,<track_number>
    - SLISt:SIZE?
"""  # noqa: E501
from typing import Dict, Optional, TYPE_CHECKING

from .._helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdReadWithArguments,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SlistSize(SCPICmdRead):
    """The ``SLISt:SIZE`` command.

    **Description:**
        - This command returns the number sequences in sequence list.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SIZE?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SIZE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SIZE?
    """


class SlistSequenceWmusage(SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:WMUSage`` command.

    **Description:**
        - This command returns the total waveform memory usage (in sample points) for the specified
          sequence track for the named sequence.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``SLISt:SEQuence:WMUSage? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:WMUSage? argument`` query and raise an AssertionError if the returned
          value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:WMUSage? <sequence_name>,<track_number>
    """


class SlistSequenceTstamp(SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:TSTamp`` command.

    **Description:**
        - This command returns the timestamp of the named sequence. Every sequence has a timestamp
          that indicates when the sequence was created or last modified.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``SLISt:SEQuence:TSTamp? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:TSTamp? argument`` query and raise an AssertionError if the returned
          value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:TSTamp? <sequence_name>
    """


class SlistSequenceTrackMax(SCPICmdRead):
    """The ``SLISt:SEQuence:TRACk:MAX`` command.

    **Description:**
        - This command returns the maximum number of tracks allowed in a sequence

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:TRACk:MAX?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:TRACk:MAX?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:TRACk:MAX?
    """


class SlistSequenceTrack(SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:TRACk`` command.

    **Description:**
        - This command returns the number of tracks defined in the specified sequence.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``SLISt:SEQuence:TRACk? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:TRACk? argument`` query and raise an AssertionError if the returned value
          does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:TRACk? <sequence_name>

    Properties:
        - ``.max``: The ``SLISt:SEQuence:TRACk:MAX`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._max = SlistSequenceTrackMax(device, f"{self._cmd_syntax}:MAX")

    @property
    def max(self) -> SlistSequenceTrackMax:
        """Return the ``SLISt:SEQuence:TRACk:MAX`` command.

        **Description:**
            - This command returns the maximum number of tracks allowed in a sequence

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:TRACk:MAX?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:TRACk:MAX?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:TRACk:MAX?
        """
        return self._max


class SlistSequenceStepItemWinput(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:WINPut`` command.

    **Description:**
        - This command sets or returns the trigger source for the wait input state for a step. Send
          a trigger signal in one of the following ways: Use an external trigger signal. Push the
          Force Trigger button on the front panel. Send the TRG or ``TRIGGER:IMMEDIATE`` remote
          commands. Use the Internal Trigger.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:WINPut value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:WINPut <sequence_name>, {ATRigger|BTRigger|ITRigger|OFF}? <sequence_name>

    **Info:**
        - ``ATRigger`` - This enables the sequencer to move due to a trigger event from the A
          Trigger Input connector or the A Force Trigger front panel button. BTRigger - This enables
          the sequencer to move due to a trigger event from the B Trigger Input connector or the B
          Force Trigger front panel button. ITRigger - This enables the sequencer to move due to an
          Internal Trigger event. OFF - Disables the wait for trigger event, allowing the
          waveforms(s) of this step to be played immediately.
    """  # noqa: E501


class SlistSequenceStepItemTflagItemDflag(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag`` command.

    **Description:**
        - This command sets or returns the Flag D value of the track in a sequence step.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

    **Info:**
        - ``NCHange`` - The flag state continues to be in the state is defined in the previous step
          Default value. HIGH - The flag signal transitions to the high state. LOW - The flag signal
          transitions to the low state. TOGGle - The flag signal transitions to the high state if
          the previous step defined the flag to be in the low state and vice versa. PULSe - The flag
          signal outputs a pulse signal of a fixed width.
    """  # noqa: E501


class SlistSequenceStepItemTflagItemCflag(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag`` command.

    **Description:**
        - This command sets or returns the Flag C value of the track in a sequence step.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

    **Info:**
        - ``NCHange`` - The flag state continues to be in the state is defined in the previous step
          Default value. HIGH - The flag signal transitions to the high state. LOW - The flag signal
          transitions to the low state. TOGGle - The flag signal transitions to the high state if
          the previous step defined the flag to be in the low state and vice versa. PULSe - The flag
          signal outputs a pulse signal of a fixed width.
    """  # noqa: E501


class SlistSequenceStepItemTflagItemBflag(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag`` command.

    **Description:**
        - This command sets or returns the Flag B value of the track in a sequence step.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

    **Info:**
        - ``NCHange`` - The flag state continues to be in the state is defined in the previous step
          Default value. HIGH - The flag signal transitions to the high state. LOW - The flag signal
          transitions to the low state. TOGGle - The flag signal transitions to the high state if
          the previous step defined the flag to be in the low state and vice versa. PULSe - The flag
          signal outputs a pulse signal of a fixed width.
    """  # noqa: E501


class SlistSequenceStepItemTflagItemAflag(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag`` command.

    **Description:**
        - This command sets or returns the Flag A value of the track in a sequence step.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

    **Info:**
        - ``NCHange`` - The flag state continues to be in the state is defined in the previous step
          Default value. HIGH - The flag signal transitions to the high state. LOW - The flag signal
          transitions to the low state. TOGGle - The flag signal transitions to the high state if
          the previous step defined the flag to be in the low state and vice versa. PULSe - The flag
          signal outputs a pulse signal of a fixed width.
    """  # noqa: E501


class SlistSequenceStepItemTflagItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SLISt:SEQuence:STEP[n]:TFLag[m]`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]:TFLag[m]?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]:TFLag[m]?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.aflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag`` command.
        - ``.bflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag`` command.
        - ``.cflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag`` command.
        - ``.dflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._aflag = SlistSequenceStepItemTflagItemAflag(device, f"{self._cmd_syntax}:AFLag")
        self._bflag = SlistSequenceStepItemTflagItemBflag(device, f"{self._cmd_syntax}:BFLag")
        self._cflag = SlistSequenceStepItemTflagItemCflag(device, f"{self._cmd_syntax}:CFLag")
        self._dflag = SlistSequenceStepItemTflagItemDflag(device, f"{self._cmd_syntax}:DFLag")

    @property
    def aflag(self) -> SlistSequenceStepItemTflagItemAflag:
        """Return the ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag`` command.

        **Description:**
            - This command sets or returns the Flag A value of the track in a sequence step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

        **Info:**
            - ``NCHange`` - The flag state continues to be in the state is defined in the previous
              step Default value. HIGH - The flag signal transitions to the high state. LOW - The
              flag signal transitions to the low state. TOGGle - The flag signal transitions to the
              high state if the previous step defined the flag to be in the low state and vice
              versa. PULSe - The flag signal outputs a pulse signal of a fixed width.
        """  # noqa: E501
        return self._aflag

    @property
    def bflag(self) -> SlistSequenceStepItemTflagItemBflag:
        """Return the ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag`` command.

        **Description:**
            - This command sets or returns the Flag B value of the track in a sequence step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

        **Info:**
            - ``NCHange`` - The flag state continues to be in the state is defined in the previous
              step Default value. HIGH - The flag signal transitions to the high state. LOW - The
              flag signal transitions to the low state. TOGGle - The flag signal transitions to the
              high state if the previous step defined the flag to be in the low state and vice
              versa. PULSe - The flag signal outputs a pulse signal of a fixed width.
        """  # noqa: E501
        return self._bflag

    @property
    def cflag(self) -> SlistSequenceStepItemTflagItemCflag:
        """Return the ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag`` command.

        **Description:**
            - This command sets or returns the Flag C value of the track in a sequence step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

        **Info:**
            - ``NCHange`` - The flag state continues to be in the state is defined in the previous
              step Default value. HIGH - The flag signal transitions to the high state. LOW - The
              flag signal transitions to the low state. TOGGle - The flag signal transitions to the
              high state if the previous step defined the flag to be in the low state and vice
              versa. PULSe - The flag signal outputs a pulse signal of a fixed width.
        """  # noqa: E501
        return self._cflag

    @property
    def dflag(self) -> SlistSequenceStepItemTflagItemDflag:
        """Return the ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag`` command.

        **Description:**
            - This command sets or returns the Flag D value of the track in a sequence step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag <sequence_name>, {NCHange|HIGH|LOW|TOGGle|PULSe}? <sequence_name>

        **Info:**
            - ``NCHange`` - The flag state continues to be in the state is defined in the previous
              step Default value. HIGH - The flag signal transitions to the high state. LOW - The
              flag signal transitions to the low state. TOGGle - The flag signal transitions to the
              high state if the previous step defined the flag to be in the low state and vice
              versa. PULSe - The flag signal outputs a pulse signal of a fixed width.
        """  # noqa: E501
        return self._dflag


class SlistSequenceStepItemTassetItemWaveform(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform`` command.

    **Description:**
        - This command assigns a waveform for a specific sequence's step and track. This waveform is
          played whenever the playing sequence reaches this step. A track in a sequence is assigned
          to a channel with the command [SOURce[n]]``:CASSET:SEQ``.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform <sequence_name>, <waveform_name>
    """


class SlistSequenceStepItemTassetItemType(SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE`` command.

    **Description:**
        - This command returns the type of asset assigned at the step and track for a specified
          sequence. The types of assets are waveform and subsequence.

    **Usage:**
        - Using the ``.query(argument)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? argument`` query and raise an AssertionError if
          the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? <sequence_name>
    """


class SlistSequenceStepItemTassetItem(ValidatedDynamicNumberCmd, SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:STEP[n]:TASSet[m]`` command.

    **Description:**
        - This command returns the name of the waveform or subsequence at the specified sequence's
          step number and track asset value. Waveform or subsequence can be distinguished by the
          ``SLIST:SEQUENCE:STEPN:TASSETM:TYPE`` query.

    **Usage:**
        - Using the ``.query(argument)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet[m]? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet[m]? argument`` query and raise an AssertionError if the
          returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TASSet[m]? <sequence_name>

    Properties:
        - ``.type``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE`` command.
        - ``.waveform``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._type = SlistSequenceStepItemTassetItemType(device, f"{self._cmd_syntax}:TYPE")
        self._waveform = SlistSequenceStepItemTassetItemWaveform(
            device, f"{self._cmd_syntax}:WAVeform"
        )

    @property
    def type(self) -> SlistSequenceStepItemTassetItemType:
        """Return the ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE`` command.

        **Description:**
            - This command returns the type of asset assigned at the step and track for a specified
              sequence. The types of assets are waveform and subsequence.

        **Usage:**
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? argument`` query and raise an AssertionError
              if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE? <sequence_name>
        """
        return self._type

    @property
    def waveform(self) -> SlistSequenceStepItemTassetItemWaveform:
        """Return the ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform`` command.

        **Description:**
            - This command assigns a waveform for a specific sequence's step and track. This
              waveform is played whenever the playing sequence reaches this step. A track in a
              sequence is assigned to a channel with the command [SOURce[n]]``:CASSET:SEQ``.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform <sequence_name>, <waveform_name>
        """
        return self._waveform


class SlistSequenceStepItemTassetSequence(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence`` command.

    **Description:**
        - This command assigns a subsequence for a specific sequence's step and track.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:TASSet:SEQuence <sequence_name>, <subsequence_name>
    """


class SlistSequenceStepItemTasset(SCPICmdRead):
    """The ``SLISt:SEQuence:STEP[n]:TASSet`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]:TASSet?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]:TASSet?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.sequence``: The ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._sequence = SlistSequenceStepItemTassetSequence(device, f"{self._cmd_syntax}:SEQuence")

    @property
    def sequence(self) -> SlistSequenceStepItemTassetSequence:
        """Return the ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence`` command.

        **Description:**
            - This command assigns a subsequence for a specific sequence's step and track.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TASSet:SEQuence <sequence_name>, <subsequence_name>
        """
        return self._sequence


class SlistSequenceStepItemRcount(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:RCOunt`` command.

    **Description:**
        - This command sets or returns the repeat count, which is the number of times the assigned
          waveform(s) play before proceeding to the next step in the sequence.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:RCOunt value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:RCOunt <sequence_name>, {ONCE|INFinite|<NR1>}? <sequence_name>

    **Info:**
        - ``ONCE`` - Plays the waveform one time during this sequence step. INFinte - Plays the
          waveform continuously during this sequence step.<NR1> - Plays this waveform the selected
          number of times during this sequence step. The allowed value is between 1 and 2^20.
    """


class SlistSequenceStepItemGoto(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:GOTO`` command.

    **Description:**
        - This command sets or returns the target step for the GOTO command of the sequencer at the
          specified step. After generating the waveform(s) specified in a sequence step, the
          sequencer jumps to the step specified as the GOTO step. This is an unconditional jump. If
          the GOTO step is not specified, the sequencer moves to the next step. If the Repeat Count
          is Infinite, the specified GOTO step is not used.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:GOTO value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:GOTO <sequence_name>, {<NR1>|LAST|FIRSt|NEXT|END}? <sequence_name>

    **Info:**
        - ``LAST`` - This enables the sequencer to go to the last step in the sequence. FIRSt - This
          enables the sequencer to go to first step in the sequence. NEXT - This enables the
          sequencer to go to the next sequence step. (The ``SLISt:SEQuence:STEP``[n]``:EJUMp:STEP``
          setting is ignored.) END - This enables the sequencer to go to the end and play 0 V until
          play is stopped.
    """


class SlistSequenceStepItemEjump(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:EJUMp`` command.

    **Description:**
        - This command sets or returns the step that the specified sequence will jump to on a
          trigger event. This setting is only available if the event jump input has been selected as
          Trigger A or Trigger B for the specified step.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:EJUMp value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:EJUMp <sequence_name>, {<NR1>|NEXT|FIRSt|LAST|END}? <sequence_name>

    **Info:**
        - ``NEXT`` - This enables the sequencer to jump to the next sequence step. FIRSt - This
          enables the sequencer to jump to first step in the sequence. LAST - This enables the
          sequencer to jump to the last step in the sequence. END - This enables the sequencer to
          jump to the end and play 0 V until play is stopped.
    """


class SlistSequenceStepItemEjinput(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP[n]:EJINput`` command.

    **Description:**
        - This command sets or returns wether the sequence will jump when it receives Trigger A,
          Trigger B, Internal Trigger, or no jump at all. This is settable for every step in a
          sequence.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:EJINput value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP[n]:EJINput <sequence_name>,{ATRigger|BTRigger|OFF|ITRigger}? <sequence_name>

    **Info:**
        - ``ATRigger`` - This enables the sequencer to jump to the event of a ATRIG. BTRigger - This
          enables the sequencer to jump to the event of a BTRIG. ITRigger - This enables the
          sequencer to jump to the event of an Internal Trigger. OFF - Ignores all events, even if
          an event occurs during that step.
        - ``*RST`` sets this to OFF.
    """  # noqa: E501


#  pylint: disable=too-many-instance-attributes
class SlistSequenceStepItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SLISt:SEQuence:STEP[n]`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ejinput``: The ``SLISt:SEQuence:STEP[n]:EJINput`` command.
        - ``.ejump``: The ``SLISt:SEQuence:STEP[n]:EJUMp`` command.
        - ``.goto``: The ``SLISt:SEQuence:STEP[n]:GOTO`` command.
        - ``.rcount``: The ``SLISt:SEQuence:STEP[n]:RCOunt`` command.
        - ``.tasset``: The ``SLISt:SEQuence:STEP[n]:TASSet`` command tree.
        - ``.tassetx``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]`` command.
        - ``.tflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]`` command tree.
        - ``.winput``: The ``SLISt:SEQuence:STEP[n]:WINPut`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ejinput = SlistSequenceStepItemEjinput(device, f"{self._cmd_syntax}:EJINput")
        self._ejump = SlistSequenceStepItemEjump(device, f"{self._cmd_syntax}:EJUMp")
        self._goto = SlistSequenceStepItemGoto(device, f"{self._cmd_syntax}:GOTO")
        self._rcount = SlistSequenceStepItemRcount(device, f"{self._cmd_syntax}:RCOunt")
        self._tasset = SlistSequenceStepItemTasset(device, f"{self._cmd_syntax}:TASSet")
        self._tassetx: Dict[int, SlistSequenceStepItemTassetItem] = DefaultDictPassKeyToFactory(
            lambda x: SlistSequenceStepItemTassetItem(device, f"{self._cmd_syntax}:TASSet{x}")
        )
        self._tflag: Dict[int, SlistSequenceStepItemTflagItem] = DefaultDictPassKeyToFactory(
            lambda x: SlistSequenceStepItemTflagItem(device, f"{self._cmd_syntax}:TFLag{x}")
        )
        self._winput = SlistSequenceStepItemWinput(device, f"{self._cmd_syntax}:WINPut")

    @property
    def ejinput(self) -> SlistSequenceStepItemEjinput:
        """Return the ``SLISt:SEQuence:STEP[n]:EJINput`` command.

        **Description:**
            - This command sets or returns wether the sequence will jump when it receives Trigger A,
              Trigger B, Internal Trigger, or no jump at all. This is settable for every step in a
              sequence.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:EJINput value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:EJINput <sequence_name>,{ATRigger|BTRigger|OFF|ITRigger}? <sequence_name>

        **Info:**
            - ``ATRigger`` - This enables the sequencer to jump to the event of a ATRIG. BTRigger -
              This enables the sequencer to jump to the event of a BTRIG. ITRigger - This enables
              the sequencer to jump to the event of an Internal Trigger. OFF - Ignores all events,
              even if an event occurs during that step.
            - ``*RST`` sets this to OFF.
        """  # noqa: E501
        return self._ejinput

    @property
    def ejump(self) -> SlistSequenceStepItemEjump:
        """Return the ``SLISt:SEQuence:STEP[n]:EJUMp`` command.

        **Description:**
            - This command sets or returns the step that the specified sequence will jump to on a
              trigger event. This setting is only available if the event jump input has been
              selected as Trigger A or Trigger B for the specified step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:EJUMp value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:EJUMp <sequence_name>, {<NR1>|NEXT|FIRSt|LAST|END}? <sequence_name>

        **Info:**
            - ``NEXT`` - This enables the sequencer to jump to the next sequence step. FIRSt - This
              enables the sequencer to jump to first step in the sequence. LAST - This enables the
              sequencer to jump to the last step in the sequence. END - This enables the sequencer
              to jump to the end and play 0 V until play is stopped.
        """  # noqa: E501
        return self._ejump

    @property
    def goto(self) -> SlistSequenceStepItemGoto:
        """Return the ``SLISt:SEQuence:STEP[n]:GOTO`` command.

        **Description:**
            - This command sets or returns the target step for the GOTO command of the sequencer at
              the specified step. After generating the waveform(s) specified in a sequence step, the
              sequencer jumps to the step specified as the GOTO step. This is an unconditional jump.
              If the GOTO step is not specified, the sequencer moves to the next step. If the Repeat
              Count is Infinite, the specified GOTO step is not used.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP[n]:GOTO value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:GOTO <sequence_name>, {<NR1>|LAST|FIRSt|NEXT|END}? <sequence_name>

        **Info:**
            - ``LAST`` - This enables the sequencer to go to the last step in the sequence. FIRSt -
              This enables the sequencer to go to first step in the sequence. NEXT - This enables
              the sequencer to go to the next sequence step. (The
              ``SLISt:SEQuence:STEP``[n]``:EJUMp:STEP`` setting is ignored.) END - This enables the
              sequencer to go to the end and play 0 V until play is stopped.
        """  # noqa: E501
        return self._goto

    @property
    def rcount(self) -> SlistSequenceStepItemRcount:
        """Return the ``SLISt:SEQuence:STEP[n]:RCOunt`` command.

        **Description:**
            - This command sets or returns the repeat count, which is the number of times the
              assigned waveform(s) play before proceeding to the next step in the sequence.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:RCOunt value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:RCOunt <sequence_name>, {ONCE|INFinite|<NR1>}? <sequence_name>

        **Info:**
            - ``ONCE`` - Plays the waveform one time during this sequence step. INFinte - Plays the
              waveform continuously during this sequence step.<NR1> - Plays this waveform the
              selected number of times during this sequence step. The allowed value is between 1 and
              2^20.
        """
        return self._rcount

    @property
    def tasset(self) -> SlistSequenceStepItemTasset:
        """Return the ``SLISt:SEQuence:STEP[n]:TASSet`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]:TASSet?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]:TASSet?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.sequence``: The ``SLISt:SEQuence:STEP[n]:TASSet:SEQuence`` command.
        """
        return self._tasset

    @property
    def tassetx(self) -> Dict[int, SlistSequenceStepItemTassetItem]:
        """Return the ``SLISt:SEQuence:STEP[n]:TASSet[m]`` command.

        **Description:**
            - This command returns the name of the waveform or subsequence at the specified
              sequence's step number and track asset value. Waveform or subsequence can be
              distinguished by the ``SLIST:SEQUENCE:STEPN:TASSETM:TYPE`` query.

        **Usage:**
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet[m]? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:TASSet[m]? argument`` query and raise an AssertionError if
              the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:TASSet[m]? <sequence_name>

        Sub-properties:
            - ``.type``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]:TYPE`` command.
            - ``.waveform``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]:WAVeform`` command.
        """
        return self._tassetx

    @property
    def tflag(self) -> Dict[int, SlistSequenceStepItemTflagItem]:
        """Return the ``SLISt:SEQuence:STEP[n]:TFLag[m]`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]:TFLag[m]?``
              query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]:TFLag[m]?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.aflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:AFLag`` command.
            - ``.bflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:BFLag`` command.
            - ``.cflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:CFLag`` command.
            - ``.dflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]:DFLag`` command.
        """
        return self._tflag

    @property
    def winput(self) -> SlistSequenceStepItemWinput:
        """Return the ``SLISt:SEQuence:STEP[n]:WINPut`` command.

        **Description:**
            - This command sets or returns the trigger source for the wait input state for a step.
              Send a trigger signal in one of the following ways: Use an external trigger signal.
              Push the Force Trigger button on the front panel. Send the TRG or
              ``TRIGGER:IMMEDIATE`` remote commands. Use the Internal Trigger.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:STEP[n]:WINPut value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP[n]:WINPut <sequence_name>, {ATRigger|BTRigger|ITRigger|OFF}? <sequence_name>

        **Info:**
            - ``ATRigger`` - This enables the sequencer to move due to a trigger event from the A
              Trigger Input connector or the A Force Trigger front panel button. BTRigger - This
              enables the sequencer to move due to a trigger event from the B Trigger Input
              connector or the B Force Trigger front panel button. ITRigger - This enables the
              sequencer to move due to an Internal Trigger event. OFF - Disables the wait for
              trigger event, allowing the waveforms(s) of this step to be played immediately.
        """  # noqa: E501
        return self._winput


class SlistSequenceStepRcountMax(SCPICmdRead):
    """The ``SLISt:SEQuence:STEP:RCOunt:MAX`` command.

    **Description:**
        - This command returns the maximum number of repeats allowed for a step in a sequence.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:RCOunt:MAX?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:RCOunt:MAX?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP:RCOunt:MAX?
    """


class SlistSequenceStepRcount(SCPICmdRead):
    """The ``SLISt:SEQuence:STEP:RCOunt`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:RCOunt?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:RCOunt?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.max``: The ``SLISt:SEQuence:STEP:RCOunt:MAX`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._max = SlistSequenceStepRcountMax(device, f"{self._cmd_syntax}:MAX")

    @property
    def max(self) -> SlistSequenceStepRcountMax:
        """Return the ``SLISt:SEQuence:STEP:RCOunt:MAX`` command.

        **Description:**
            - This command returns the maximum number of repeats allowed for a step in a sequence.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:RCOunt:MAX?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:RCOunt:MAX?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP:RCOunt:MAX?
        """
        return self._max


class SlistSequenceStepMax(SCPICmdRead):
    """The ``SLISt:SEQuence:STEP:MAX`` command.

    **Description:**
        - This command returns the maximum number of steps allowed in a sequence.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:MAX?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:MAX?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP:MAX?
    """


class SlistSequenceStepAdd(SCPICmdWrite):
    """The ``SLISt:SEQuence:STEP:ADD`` command.

    **Description:**
        - This command adds steps to the named sequence. If the specified location is occupied, the
          step(s) are inserted prior to the specified step. If the specified location is the first
          unoccupied step in the sequence, the step(s) are appended to the sequence. If the
          specified location would result in a gap within the sequence, steps are added to bridge
          the gap in addition to the number of steps specified to add. For example, if you have a
          sequence with 25 steps, and you specify to add 5 steps beginning at location 30, steps
          will be added to fill the gap between steps 25 and 30.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP:ADD value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:STEP:ADD <sequence_name>,<location>[,<steps to add>]
    """


class SlistSequenceStep(SCPICmdRead):
    """The ``SLISt:SEQuence:STEP`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.add``: The ``SLISt:SEQuence:STEP:ADD`` command.
        - ``.max``: The ``SLISt:SEQuence:STEP:MAX`` command.
        - ``.rcount``: The ``SLISt:SEQuence:STEP:RCOunt`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._add = SlistSequenceStepAdd(device, f"{self._cmd_syntax}:ADD")
        self._max = SlistSequenceStepMax(device, f"{self._cmd_syntax}:MAX")
        self._rcount = SlistSequenceStepRcount(device, f"{self._cmd_syntax}:RCOunt")

    @property
    def add(self) -> SlistSequenceStepAdd:
        """Return the ``SLISt:SEQuence:STEP:ADD`` command.

        **Description:**
            - This command adds steps to the named sequence. If the specified location is occupied,
              the step(s) are inserted prior to the specified step. If the specified location is the
              first unoccupied step in the sequence, the step(s) are appended to the sequence. If
              the specified location would result in a gap within the sequence, steps are added to
              bridge the gap in addition to the number of steps specified to add. For example, if
              you have a sequence with 25 steps, and you specify to add 5 steps beginning at
              location 30, steps will be added to fill the gap between steps 25 and 30.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:STEP:ADD value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP:ADD <sequence_name>,<location>[,<steps to add>]
        """
        return self._add

    @property
    def max(self) -> SlistSequenceStepMax:
        """Return the ``SLISt:SEQuence:STEP:MAX`` command.

        **Description:**
            - This command returns the maximum number of steps allowed in a sequence.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:MAX?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:MAX?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:STEP:MAX?
        """
        return self._max

    @property
    def rcount(self) -> SlistSequenceStepRcount:
        """Return the ``SLISt:SEQuence:STEP:RCOunt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP:RCOunt?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP:RCOunt?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.max``: The ``SLISt:SEQuence:STEP:RCOunt:MAX`` command.
        """
        return self._rcount


class SlistSequenceSrate(SCPICmdWrite):
    """The ``SLISt:SEQuence:SRATe`` command.

    **Description:**
        - The command sets or returns the Recommended Sampling Rate of the specified sequence.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:SRATe value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:SRATe <sequence_name>,<sample_rate>? <sequence_name>
    """


class SlistSequenceRflag(SCPICmdWrite):
    """The ``SLISt:SEQuence:RFLag`` command.

    **Description:**
        - This command sets or returns the Enable Flag Repeat value for the sequence. If the value
          is ON, then the flags will change each time that the step plays out. For example if Wfm1
          is at a step in Sequence with repeat 2 and one of the flags is set to Toggle, then the
          flag state will toggle twice at this step if the Enable Flag Repeat value is ON.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:RFLag value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:RFLag <sequence_name>, {0|1|OFF|ON}? <sequence_name>
    """


class SlistSequenceOffset(SCPICmdWrite):
    """The ``SLISt:SEQuence:OFFSet`` command.

    **Description:**
        - The command sets or returns the Recommended Offset of the specified sequence.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:OFFSet value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:OFFSet <sequence_name>,<offset>? <sequence_name>
    """


class SlistSequenceNew(SCPICmdWrite):
    """The ``SLISt:SEQuence:NEW`` command.

    **Description:**
        - This command creates a new sequence with the selected name, number of steps, and number of
          tracks.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:NEW value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:NEW <sequence_name>,<number_of_steps>[,<number_of_tracks>]
    """


class SlistSequenceLength(SCPICmdReadWithArguments):
    """The ``SLISt:SEQuence:LENGth`` command.

    **Description:**
        - This command returns the total number of steps in the named sequence.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``SLISt:SEQuence:LENGth? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``SLISt:SEQuence:LENGth? argument`` query and raise an AssertionError if the returned
          value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:LENGth? <sequence_name>
    """


class SlistSequenceFrequency(SCPICmdWrite):
    """The ``SLISt:SEQuence:FREQuency`` command.

    **Description:**
        - The command sets or returns the recommended frequency of the specified sequence when the
          sequence contains IQ waveforms.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:FREQuency value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:FREQuency <seq_name>,<frequency>? <seq_name>
    """


class SlistSequenceEventPjumpSize(SCPICmdRead):
    """The ``SLISt:SEQuence:EVENt:PJUMp:SIZE`` command.

    **Description:**
        - This command returns the maximum number of entries in the pattern jump table.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt:PJUMp:SIZE?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt:PJUMp:SIZE?``
          query and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:EVENt:PJUMp:SIZE?
    """


class SlistSequenceEventPjumpEnable(SCPICmdWrite):
    """The ``SLISt:SEQuence:EVENt:PJUMp:ENABle`` command.

    **Description:**
        - This command sets or returns the Event Pattern Jump state (enabled or disabled) for the
          named sequence. When enabled, the data at the Pattern Jump In connector can be strobed in,
          causing a sequence to jump to a defined step. The sequence and step are defined with the
          command ``SLIST:SEQUENCE:EVENT:PJUMP:DEFINE``.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:EVENt:PJUMp:ENABle value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:EVENt:PJUMp:ENABle <sequence_name>, {0|1|OFF|ON}? <sequence_name>

    **Info:**
        - ``*RST`` sets this to 0.
    """


class SlistSequenceEventPjumpDefine(SCPICmdWrite):
    """The ``SLISt:SEQuence:EVENt:PJUMp:DEFine`` command.

    **Description:**
        - This command associates an event pattern with the jump-to-step for Pattern Jump. The query
          returns the jump step associated to the specified pattern. The event pattern is read from
          the Pattern Jump In connector on the rear panel. Eight bits of data and a strobe are
          required. When the strobed event pattern is received, an event pattern jump is created,
          moving the sequence to the step defined in this command.

    **Usage:**
        - Using the ``.write(value)`` method will send the
          ``SLISt:SEQuence:EVENt:PJUMp:DEFine value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:EVENt:PJUMp:DEFine <sequence_name>, <pattern>, <jump_step>? <sequence_name>, <pattern>
    """  # noqa: E501


class SlistSequenceEventPjump(SCPICmdRead):
    """The ``SLISt:SEQuence:EVENt:PJUMp`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt:PJUMp?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt:PJUMp?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.define``: The ``SLISt:SEQuence:EVENt:PJUMp:DEFine`` command.
        - ``.enable``: The ``SLISt:SEQuence:EVENt:PJUMp:ENABle`` command.
        - ``.size``: The ``SLISt:SEQuence:EVENt:PJUMp:SIZE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._define = SlistSequenceEventPjumpDefine(device, f"{self._cmd_syntax}:DEFine")
        self._enable = SlistSequenceEventPjumpEnable(device, f"{self._cmd_syntax}:ENABle")
        self._size = SlistSequenceEventPjumpSize(device, f"{self._cmd_syntax}:SIZE")

    @property
    def define(self) -> SlistSequenceEventPjumpDefine:
        """Return the ``SLISt:SEQuence:EVENt:PJUMp:DEFine`` command.

        **Description:**
            - This command associates an event pattern with the jump-to-step for Pattern Jump. The
              query returns the jump step associated to the specified pattern. The event pattern is
              read from the Pattern Jump In connector on the rear panel. Eight bits of data and a
              strobe are required. When the strobed event pattern is received, an event pattern jump
              is created, moving the sequence to the step defined in this command.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:EVENt:PJUMp:DEFine value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:EVENt:PJUMp:DEFine <sequence_name>, <pattern>, <jump_step>? <sequence_name>, <pattern>
        """  # noqa: E501
        return self._define

    @property
    def enable(self) -> SlistSequenceEventPjumpEnable:
        """Return the ``SLISt:SEQuence:EVENt:PJUMp:ENABle`` command.

        **Description:**
            - This command sets or returns the Event Pattern Jump state (enabled or disabled) for
              the named sequence. When enabled, the data at the Pattern Jump In connector can be
              strobed in, causing a sequence to jump to a defined step. The sequence and step are
              defined with the command ``SLIST:SEQUENCE:EVENT:PJUMP:DEFINE``.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:EVENt:PJUMp:ENABle value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:EVENt:PJUMp:ENABle <sequence_name>, {0|1|OFF|ON}? <sequence_name>

        **Info:**
            - ``*RST`` sets this to 0.
        """
        return self._enable

    @property
    def size(self) -> SlistSequenceEventPjumpSize:
        """Return the ``SLISt:SEQuence:EVENt:PJUMp:SIZE`` command.

        **Description:**
            - This command returns the maximum number of entries in the pattern jump table.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt:PJUMp:SIZE?``
              query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt:PJUMp:SIZE?``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:EVENt:PJUMp:SIZE?
        """
        return self._size


class SlistSequenceEventJtiming(SCPICmdWrite):
    """The ``SLISt:SEQuence:EVENt:JTIMing`` command.

    **Description:**
        - This command sets or returns the condition of when the sequencer jumps upon a logic event,
          pattern jump, or software forced jump. The jump can occur immediately or at the end of the
          current sequence step.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:EVENt:JTIMing value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:EVENt:JTIMing <sequence_name>, {END|IMMediate}? <sequence_name>

    **Info:**
        - ``END`` - on receiving an event, wait until the end of current step before jumping to
          specified event jump step IMMediate - on receiving an event, immediately jump to specified
          event jump step.
    """


class SlistSequenceEvent(SCPICmdRead):
    """The ``SLISt:SEQuence:EVENt`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.jtiming``: The ``SLISt:SEQuence:EVENt:JTIMing`` command.
        - ``.pjump``: The ``SLISt:SEQuence:EVENt:PJUMp`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._jtiming = SlistSequenceEventJtiming(device, f"{self._cmd_syntax}:JTIMing")
        self._pjump = SlistSequenceEventPjump(device, f"{self._cmd_syntax}:PJUMp")

    @property
    def jtiming(self) -> SlistSequenceEventJtiming:
        """Return the ``SLISt:SEQuence:EVENt:JTIMing`` command.

        **Description:**
            - This command sets or returns the condition of when the sequencer jumps upon a logic
              event, pattern jump, or software forced jump. The jump can occur immediately or at the
              end of the current sequence step.

        **Usage:**
            - Using the ``.write(value)`` method will send the
              ``SLISt:SEQuence:EVENt:JTIMing value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:EVENt:JTIMing <sequence_name>, {END|IMMediate}? <sequence_name>

        **Info:**
            - ``END`` - on receiving an event, wait until the end of current step before jumping to
              specified event jump step IMMediate - on receiving an event, immediately jump to
              specified event jump step.
        """
        return self._jtiming

    @property
    def pjump(self) -> SlistSequenceEventPjump:
        """Return the ``SLISt:SEQuence:EVENt:PJUMp`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt:PJUMp?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt:PJUMp?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.define``: The ``SLISt:SEQuence:EVENt:PJUMp:DEFine`` command.
            - ``.enable``: The ``SLISt:SEQuence:EVENt:PJUMp:ENABle`` command.
            - ``.size``: The ``SLISt:SEQuence:EVENt:PJUMp:SIZE`` command.
        """
        return self._pjump


class SlistSequenceDelete(SCPICmdWrite):
    """The ``SLISt:SEQuence:DELete`` command.

    **Description:**
        - This command deletes a specific sequence or all sequences from the sequence list.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:DELete value`` command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:DELete {<sequence_name>|ALL}
    """


class SlistSequenceAmplitude(SCPICmdWrite):
    """The ``SLISt:SEQuence:AMPLitude`` command.

    **Description:**
        - The command sets or returns the Recommended Amplitude (peak-to-peak) of the specified
          sequence.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:AMPLitude value``
          command.

    **SCPI Syntax:**

    ::

        - SLISt:SEQuence:AMPLitude <sequence_name>,<amplitude>? <sequence_name>
    """


#  pylint: disable=too-many-instance-attributes
class SlistSequence(SCPICmdRead):
    """The ``SLISt:SEQuence`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt:SEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.amplitude``: The ``SLISt:SEQuence:AMPLitude`` command.
        - ``.delete``: The ``SLISt:SEQuence:DELete`` command.
        - ``.event``: The ``SLISt:SEQuence:EVENt`` command tree.
        - ``.frequency``: The ``SLISt:SEQuence:FREQuency`` command.
        - ``.length``: The ``SLISt:SEQuence:LENGth`` command.
        - ``.new``: The ``SLISt:SEQuence:NEW`` command.
        - ``.offset``: The ``SLISt:SEQuence:OFFSet`` command.
        - ``.rflag``: The ``SLISt:SEQuence:RFLag`` command.
        - ``.srate``: The ``SLISt:SEQuence:SRATe`` command.
        - ``.step``: The ``SLISt:SEQuence:STEP`` command tree.
        - ``.stepx``: The ``SLISt:SEQuence:STEP[n]`` command tree.
        - ``.track``: The ``SLISt:SEQuence:TRACk`` command.
        - ``.tstamp``: The ``SLISt:SEQuence:TSTamp`` command.
        - ``.wmusage``: The ``SLISt:SEQuence:WMUSage`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._amplitude = SlistSequenceAmplitude(device, f"{self._cmd_syntax}:AMPLitude")
        self._delete = SlistSequenceDelete(device, f"{self._cmd_syntax}:DELete")
        self._event = SlistSequenceEvent(device, f"{self._cmd_syntax}:EVENt")
        self._frequency = SlistSequenceFrequency(device, f"{self._cmd_syntax}:FREQuency")
        self._length = SlistSequenceLength(device, f"{self._cmd_syntax}:LENGth")
        self._new = SlistSequenceNew(device, f"{self._cmd_syntax}:NEW")
        self._offset = SlistSequenceOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._rflag = SlistSequenceRflag(device, f"{self._cmd_syntax}:RFLag")
        self._srate = SlistSequenceSrate(device, f"{self._cmd_syntax}:SRATe")
        self._step = SlistSequenceStep(device, f"{self._cmd_syntax}:STEP")
        self._stepx: Dict[int, SlistSequenceStepItem] = DefaultDictPassKeyToFactory(
            lambda x: SlistSequenceStepItem(device, f"{self._cmd_syntax}:STEP{x}")
        )
        self._track = SlistSequenceTrack(device, f"{self._cmd_syntax}:TRACk")
        self._tstamp = SlistSequenceTstamp(device, f"{self._cmd_syntax}:TSTamp")
        self._wmusage = SlistSequenceWmusage(device, f"{self._cmd_syntax}:WMUSage")

    @property
    def amplitude(self) -> SlistSequenceAmplitude:
        """Return the ``SLISt:SEQuence:AMPLitude`` command.

        **Description:**
            - The command sets or returns the Recommended Amplitude (peak-to-peak) of the specified
              sequence.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:AMPLitude value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:AMPLitude <sequence_name>,<amplitude>? <sequence_name>
        """
        return self._amplitude

    @property
    def delete(self) -> SlistSequenceDelete:
        """Return the ``SLISt:SEQuence:DELete`` command.

        **Description:**
            - This command deletes a specific sequence or all sequences from the sequence list.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:DELete value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:DELete {<sequence_name>|ALL}
        """
        return self._delete

    @property
    def event(self) -> SlistSequenceEvent:
        """Return the ``SLISt:SEQuence:EVENt`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:EVENt?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:EVENt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.jtiming``: The ``SLISt:SEQuence:EVENt:JTIMing`` command.
            - ``.pjump``: The ``SLISt:SEQuence:EVENt:PJUMp`` command tree.
        """
        return self._event

    @property
    def frequency(self) -> SlistSequenceFrequency:
        """Return the ``SLISt:SEQuence:FREQuency`` command.

        **Description:**
            - The command sets or returns the recommended frequency of the specified sequence when
              the sequence contains IQ waveforms.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:FREQuency value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:FREQuency <seq_name>,<frequency>? <seq_name>
        """
        return self._frequency

    @property
    def length(self) -> SlistSequenceLength:
        """Return the ``SLISt:SEQuence:LENGth`` command.

        **Description:**
            - This command returns the total number of steps in the named sequence.

        **Usage:**
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SEQuence:LENGth? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:LENGth? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:LENGth? <sequence_name>
        """
        return self._length

    @property
    def new(self) -> SlistSequenceNew:
        """Return the ``SLISt:SEQuence:NEW`` command.

        **Description:**
            - This command creates a new sequence with the selected name, number of steps, and
              number of tracks.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:NEW value`` command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:NEW <sequence_name>,<number_of_steps>[,<number_of_tracks>]
        """
        return self._new

    @property
    def offset(self) -> SlistSequenceOffset:
        """Return the ``SLISt:SEQuence:OFFSet`` command.

        **Description:**
            - The command sets or returns the Recommended Offset of the specified sequence.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:OFFSet value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:OFFSet <sequence_name>,<offset>? <sequence_name>
        """
        return self._offset

    @property
    def rflag(self) -> SlistSequenceRflag:
        """Return the ``SLISt:SEQuence:RFLag`` command.

        **Description:**
            - This command sets or returns the Enable Flag Repeat value for the sequence. If the
              value is ON, then the flags will change each time that the step plays out. For example
              if Wfm1 is at a step in Sequence with repeat 2 and one of the flags is set to Toggle,
              then the flag state will toggle twice at this step if the Enable Flag Repeat value is
              ON.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:RFLag value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:RFLag <sequence_name>, {0|1|OFF|ON}? <sequence_name>
        """
        return self._rflag

    @property
    def srate(self) -> SlistSequenceSrate:
        """Return the ``SLISt:SEQuence:SRATe`` command.

        **Description:**
            - The command sets or returns the Recommended Sampling Rate of the specified sequence.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SLISt:SEQuence:SRATe value``
              command.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:SRATe <sequence_name>,<sample_rate>? <sequence_name>
        """
        return self._srate

    @property
    def step(self) -> SlistSequenceStep:
        """Return the ``SLISt:SEQuence:STEP`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.add``: The ``SLISt:SEQuence:STEP:ADD`` command.
            - ``.max``: The ``SLISt:SEQuence:STEP:MAX`` command.
            - ``.rcount``: The ``SLISt:SEQuence:STEP:RCOunt`` command tree.
        """
        return self._step

    @property
    def stepx(self) -> Dict[int, SlistSequenceStepItem]:
        """Return the ``SLISt:SEQuence:STEP[n]`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence:STEP[n]?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence:STEP[n]?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.ejinput``: The ``SLISt:SEQuence:STEP[n]:EJINput`` command.
            - ``.ejump``: The ``SLISt:SEQuence:STEP[n]:EJUMp`` command.
            - ``.goto``: The ``SLISt:SEQuence:STEP[n]:GOTO`` command.
            - ``.rcount``: The ``SLISt:SEQuence:STEP[n]:RCOunt`` command.
            - ``.tasset``: The ``SLISt:SEQuence:STEP[n]:TASSet`` command tree.
            - ``.tassetx``: The ``SLISt:SEQuence:STEP[n]:TASSet[m]`` command.
            - ``.tflag``: The ``SLISt:SEQuence:STEP[n]:TFLag[m]`` command tree.
            - ``.winput``: The ``SLISt:SEQuence:STEP[n]:WINPut`` command.
        """
        return self._stepx

    @property
    def track(self) -> SlistSequenceTrack:
        """Return the ``SLISt:SEQuence:TRACk`` command.

        **Description:**
            - This command returns the number of tracks defined in the specified sequence.

        **Usage:**
            - Using the ``.query(argument)`` method will send the ``SLISt:SEQuence:TRACk? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:TRACk? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:TRACk? <sequence_name>

        Sub-properties:
            - ``.max``: The ``SLISt:SEQuence:TRACk:MAX`` command.
        """
        return self._track

    @property
    def tstamp(self) -> SlistSequenceTstamp:
        """Return the ``SLISt:SEQuence:TSTamp`` command.

        **Description:**
            - This command returns the timestamp of the named sequence. Every sequence has a
              timestamp that indicates when the sequence was created or last modified.

        **Usage:**
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SEQuence:TSTamp? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:TSTamp? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:TSTamp? <sequence_name>
        """
        return self._tstamp

    @property
    def wmusage(self) -> SlistSequenceWmusage:
        """Return the ``SLISt:SEQuence:WMUSage`` command.

        **Description:**
            - This command returns the total waveform memory usage (in sample points) for the
              specified sequence track for the named sequence.

        **Usage:**
            - Using the ``.query(argument)`` method will send the
              ``SLISt:SEQuence:WMUSage? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``SLISt:SEQuence:WMUSage? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SEQuence:WMUSage? <sequence_name>,<track_number>
        """
        return self._wmusage


class SlistName(SCPICmdReadWithArguments):
    """The ``SLISt:NAME`` command.

    **Description:**
        - This command returns the name of the sequence at the specified sequence list index.

    **Usage:**
        - Using the ``.query(argument)`` method will send the ``SLISt:NAME? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the ``SLISt:NAME? argument`` query
          and raise an AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SLISt:NAME? <sequence_list_index>
    """


class Slist(SCPICmdRead):
    """The ``SLISt`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SLISt?`` query.
        - Using the ``.verify(value)`` method will send the ``SLISt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.name``: The ``SLISt:NAME`` command.
        - ``.sequence``: The ``SLISt:SEQuence`` command tree.
        - ``.size``: The ``SLISt:SIZE`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SLISt") -> None:
        super().__init__(device, cmd_syntax)
        self._name = SlistName(device, f"{self._cmd_syntax}:NAME")
        self._sequence = SlistSequence(device, f"{self._cmd_syntax}:SEQuence")
        self._size = SlistSize(device, f"{self._cmd_syntax}:SIZE")

    @property
    def name(self) -> SlistName:
        """Return the ``SLISt:NAME`` command.

        **Description:**
            - This command returns the name of the sequence at the specified sequence list index.

        **Usage:**
            - Using the ``.query(argument)`` method will send the ``SLISt:NAME? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the ``SLISt:NAME? argument``
              query and raise an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:NAME? <sequence_list_index>
        """
        return self._name

    @property
    def sequence(self) -> SlistSequence:
        """Return the ``SLISt:SEQuence`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SEQuence?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.amplitude``: The ``SLISt:SEQuence:AMPLitude`` command.
            - ``.delete``: The ``SLISt:SEQuence:DELete`` command.
            - ``.event``: The ``SLISt:SEQuence:EVENt`` command tree.
            - ``.frequency``: The ``SLISt:SEQuence:FREQuency`` command.
            - ``.length``: The ``SLISt:SEQuence:LENGth`` command.
            - ``.new``: The ``SLISt:SEQuence:NEW`` command.
            - ``.offset``: The ``SLISt:SEQuence:OFFSet`` command.
            - ``.rflag``: The ``SLISt:SEQuence:RFLag`` command.
            - ``.srate``: The ``SLISt:SEQuence:SRATe`` command.
            - ``.step``: The ``SLISt:SEQuence:STEP`` command tree.
            - ``.stepx``: The ``SLISt:SEQuence:STEP[n]`` command tree.
            - ``.track``: The ``SLISt:SEQuence:TRACk`` command.
            - ``.tstamp``: The ``SLISt:SEQuence:TSTamp`` command.
            - ``.wmusage``: The ``SLISt:SEQuence:WMUSage`` command.
        """
        return self._sequence

    @property
    def size(self) -> SlistSize:
        """Return the ``SLISt:SIZE`` command.

        **Description:**
            - This command returns the number sequences in sequence list.

        **Usage:**
            - Using the ``.query()`` method will send the ``SLISt:SIZE?`` query.
            - Using the ``.verify(value)`` method will send the ``SLISt:SIZE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SLISt:SIZE?
        """
        return self._size



"""The setup1 commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SETUP1<x>:DATE?
    - SETUP1<x>:LABel <Qstring>
    - SETUP1<x>:TIMe?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Setup1ItemTime(SCPICmdRead):
    """The ``SETUP1<x>:TIMe`` command.

    **Description:**
        - Returns the time when the oscilloscope setup was saved for the specified channel <x>,
          where x can be 1 through 10.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP1<x>:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP1<x>:TIMe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SETUP1<x>:TIMe?
    """


class Setup1ItemLabel(SCPICmdWrite):
    """The ``SETUP1<x>:LABel`` command.

    **Description:**
        - This command specifies the setup label for the specified channel <x>, where x can be 1
          through 10.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``SETUP1<x>:LABel value`` command.

    **SCPI Syntax:**

    ::

        - SETUP1<x>:LABel <Qstring>

    **Info:**
        - ``<Qstring>`` is an alphanumeric string of characters, enclosed in quotes, that defines
          the label text for SETUP<x>. The length of the string is limited to 30 characters.
    """


class Setup1ItemDate(SCPICmdRead):
    """The ``SETUP1<x>:DATE`` command.

    **Description:**
        - Returns the date when the oscilloscope setup was saved for the specified channel <x>,
          where x can be 1 through 10.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP1<x>:DATE?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP1<x>:DATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - SETUP1<x>:DATE?
    """


class Setup1Item(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SETUP1<x>`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SETUP1<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``SETUP1<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.date``: The ``SETUP1<x>:DATE`` command.
        - ``.label``: The ``SETUP1<x>:LABel`` command.
        - ``.time``: The ``SETUP1<x>:TIMe`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SETUP1<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._date = Setup1ItemDate(device, f"{self._cmd_syntax}:DATE")
        self._label = Setup1ItemLabel(device, f"{self._cmd_syntax}:LABel")
        self._time = Setup1ItemTime(device, f"{self._cmd_syntax}:TIMe")

    @property
    def date(self) -> Setup1ItemDate:
        """Return the ``SETUP1<x>:DATE`` command.

        **Description:**
            - Returns the date when the oscilloscope setup was saved for the specified channel <x>,
              where x can be 1 through 10.

        **Usage:**
            - Using the ``.query()`` method will send the ``SETUP1<x>:DATE?`` query.
            - Using the ``.verify(value)`` method will send the ``SETUP1<x>:DATE?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SETUP1<x>:DATE?
        """
        return self._date

    @property
    def label(self) -> Setup1ItemLabel:
        """Return the ``SETUP1<x>:LABel`` command.

        **Description:**
            - This command specifies the setup label for the specified channel <x>, where x can be 1
              through 10.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``SETUP1<x>:LABel value`` command.

        **SCPI Syntax:**

        ::

            - SETUP1<x>:LABel <Qstring>

        **Info:**
            - ``<Qstring>`` is an alphanumeric string of characters, enclosed in quotes, that
              defines the label text for SETUP<x>. The length of the string is limited to 30
              characters.
        """
        return self._label

    @property
    def time(self) -> Setup1ItemTime:
        """Return the ``SETUP1<x>:TIMe`` command.

        **Description:**
            - Returns the time when the oscilloscope setup was saved for the specified channel <x>,
              where x can be 1 through 10.

        **Usage:**
            - Using the ``.query()`` method will send the ``SETUP1<x>:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``SETUP1<x>:TIMe?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - SETUP1<x>:TIMe?
        """
        return self._time



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The buffer commands module.

These commands are used in the following models:
DMM7510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - buffer.clearstats()
    - buffer.delete()
    - buffer.make()
    - buffer.save()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Buffer(BaseTSPCmd):
    """The ``buffer`` command tree.

    Constants:
        - ``.COL_ALL``: Save all data from the specified reading buffer.
        - ``.COL_BRIEF``: Save reading and relative time data from the specified reading buffer.
        - ``.COL_DISPLAY_DIGITS``: Save display digits from the specified reading buffer.
        - ``.COL_EXTRA``: Relative time and additional values if they exist (such as the sense
          voltage from a dc voltage ratio measurement).
        - ``.COL_EXTRA_RANGE``: Save sxtra value range digits from the specified reading buffer.
        - ``.COL_EXTRA_UNIT``: Save extra value units from the specified reading buffer.
        - ``.COL_EXTRA_VALUE``: Save extra values from the specified reading buffer.
        - ``.COL_INDEX``: Save index into buffer from the specified reading buffer.
        - ``.COL_LIMITS``: The status of all limits.
        - ``.COL_MATH``: Math enabled (F is math is not enabled; T if math is enabled) and relative
          time.
        - ``.COL_ORIGIN``: Save origin status from the specified reading buffer.
        - ``.COL_QUESTIONABLE``: Save questionable status from the specified reading buffer.
        - ``.COL_RANGE_DIGITS``: Save range digits from the specified reading buffer.
        - ``.COL_READING``: Save the measurement reading from the specified reading buffer.
        - ``.COL_STANDARD``: Save the relative time, reading, channel, and source value from the
          specified reading buffer.
        - ``.COL_START``: Save the status of the start group from the specified reading buffer.
        - ``.COL_STATUS``: Save the status information associated with the measurement from the
          specified reading buffer.
        - ``.COL_TERMINAL``: Save the terminal status from the specified reading buffer.
        - ``.COL_TIMESTAMP_READING``: Save the timestamp reading from the specified reading buffer.
        - ``.COL_TIME_ABSOLUTE``: Save the time when the data point was measured as an absolute from
          the specified reading buffer.
        - ``.COL_TIME_PARTS``: Save absolute time in multiple columns from the specified reading
          buffer.
        - ``.COL_TIME_RAW``: Save absolute time in seconds from the specified reading buffer.
        - ``.COL_TIME_RELATIVE``: Save the relative time when the data point was measured in seconds
          from the specified reading buffer.
        - ``.COL_UNIT``: Save the reading and the unit of measure from the specified reading buffer.
        - ``.DIGITS_3_5``: The number of digits to use for the  first measurement.
        - ``.DIGITS_4_5``: The number of digits to use for the  first measurement.
        - ``.DIGITS_5_5``: The number of digits to use for the  first measurement.
        - ``.DIGITS_6_5``: The number of digits to use for the  first measurement.
        - ``.DIGITS_7_5``: The number of digits to use for the  first measurement.
        - ``.DIGITS_8_5``: The number of digits to use for the  first measurement.
        - ``.EXPR_ADD``: Add the present and previous measurements.
        - ``.EXPR_AVERAGE``: Average the present and previous measurements.
        - ``.EXPR_DIVIDE``: Divide the present reading by the previous reading.
        - ``.EXPR_EXPONENT``: Exponent (10^present reading).
        - ``.EXPR_LOG10``: Log10 (log10(present reading)).
        - ``.EXPR_MULTIPLY``: Multiply the present and previous measurements.
        - ``.EXPR_NONE``: Remove the math expression.
        - ``.EXPR_POLY``: Polynomial where r is present reading and c is constant.
        - ``.EXPR_POWER``: Present reading raised to power of defined constant.
        - ``.EXPR_RATE``: Rate of change (present reading - previous reading)/(timestamp of present
          reading - timestamp of previous reading).
        - ``.EXPR_RECIPROCAL``: Reciprocal (1/present reading).
        - ``.EXPR_SQROOT``: Square root of the present reading.
        - ``.EXPR_SUBTRACT``: Present reading - previous reading.
        - ``.FILL_CONTINUOUS``: Fill the buffer continuously.
        - ``.FILL_ONCE``: Fill the buffer, then stop.
        - ``.OFF``: Do not log information events.
        - ``.ON``: Log information events.
        - ``.SAVE_FORMAT_TIME``: Include dates, times, and fractional seconds in the buffer.
        - ``.SAVE_RAW_TIME``: Include seconds and fractional seconds in the buffer.
        - ``.SAVE_RELATIVE_TIME``: Include relative timestamps in the buffer.
        - ``.SAVE_TIMESTAMP_TIME``: Include timestamps in the buffer.
        - ``.STAT_LIMIT1_HIGH``: Reading is above high limit 1.
        - ``.STAT_LIMIT1_LOW``: Reading is below low limit 1.
        - ``.STAT_LIMIT2_HIGH``: Reading is above high limit 2.
        - ``.STAT_LIMIT2_LOW``: Reading is below low limit 2.
        - ``.STAT_ORIGIN``: A/D converter from which reading originated; for most instruments, this
          will always be 0 (main). For digitizing instruments, can be 2 (digitize).
        - ``.STAT_QUESTIONABLE``: Measure status questionable.
        - ``.STAT_REL``: Relative offset.
        - ``.STAT_SCAN``: Scan.
        - ``.STAT_START_GROUP``: First reading in a group.
        - ``.STAT_TERMINAL``: Measure terminal, front is 1, rear is 0.
        - ``.STYLE_COMPACT``: Store readings with reduced accuracy (6.5 digits) with no formatting
          information, 1 μs accurate timestamp.
        - ``.UNIT_AMP``: Set units of measure to dc current.
        - ``.UNIT_AMP_AC``: Set units of measure to ac current.
        - ``.UNIT_CELSIUS``: Set units of measure to Celsius.
        - ``.UNIT_CUSTOM1``: Custom unit 1 (defined by buffer.unit()).
        - ``.UNIT_CUSTOM2``: Custom unit 2 (defined by buffer.unit()).
        - ``.UNIT_CUSTOM3``: Custom unit 3 (defined by buffer.unit()).
        - ``.UNIT_DAC``: Set units of measure to DAC (voltage).
        - ``.UNIT_DBM``: Set units of measure to decibel-milliwatts.
        - ``.UNIT_DECIBEL``: Set units of measure to decibels.
        - ``.UNIT_DIO``: Set units of measure to digital I/O.
        - ``.UNIT_FAHRENHEIT``: Set units of measure to Fahrenheit.
        - ``.UNIT_FARAD``: Set units of measure to capacitance.
        - ``.UNIT_HERTZ``: Set units of measure to frequency.
        - ``.UNIT_KELVIN``: Set units of measure to Kelvin.
        - ``.UNIT_NONE``: Set units of measure to no units.
        - ``.UNIT_OHM``: Set units of measure to ohms.
        - ``.UNIT_PERCENT``: Set units of measure to percent.
        - ``.UNIT_RATIO``: Set units of measure to dc voltage ratio.
        - ``.UNIT_RECIPROCAL``: Set units of measure to reciprocal.
        - ``.UNIT_SECOND``: Set units of measure to period.
        - ``.UNIT_TOT``: Set units of measure to totalizer.
        - ``.UNIT_VOLT``: Set units of measure to dc voltage.
        - ``.UNIT_VOLT_AC``: Set units of measure to ac voltage.
        - ``.UNIT_WATT``: Set units of measure to watts.
        - ``.UNIT_X``: Set units of measure to buffer.UNIT_X.

    Properties/methods:
        - ``.clearstats()``: The ``buffer.clearstats()`` function.
        - ``.delete()``: The ``buffer.delete()`` function.
        - ``.make()``: The ``buffer.make()`` function.
        - ``.save()``: The ``buffer.save()`` function.
    """

    COL_ALL = "buffer.COL_ALL"
    """str: Save all data from the specified reading buffer."""
    COL_BRIEF = "buffer.COL_BRIEF"
    """str: Save reading and relative time data from the specified reading buffer."""
    COL_DISPLAY_DIGITS = "buffer.COL_DISPLAY_DIGITS"
    """str: Save display digits from the specified reading buffer."""
    COL_EXTRA = "buffer.COL_EXTRA"
    """str: Relative time and additional values if they exist (such as the sense voltage from a dc voltage ratio measurement)."""  # noqa: E501
    COL_EXTRA_RANGE = "buffer.COL_EXTRA_RANGE"
    """str: Save sxtra value range digits from the specified reading buffer."""
    COL_EXTRA_UNIT = "buffer.COL_EXTRA_UNIT"
    """str: Save extra value units from the specified reading buffer."""
    COL_EXTRA_VALUE = "buffer.COL_EXTRA_VALUE"
    """str: Save extra values from the specified reading buffer."""
    COL_INDEX = "buffer.COL_INDEX"
    """str: Save index into buffer from the specified reading buffer."""
    COL_LIMITS = "buffer.COL_LIMITS"
    """str: The status of all limits."""
    COL_MATH = "buffer.COL_MATH"
    """str: Math enabled (F is math is not enabled; T if math is enabled) and relative time."""
    COL_ORIGIN = "buffer.COL_ORIGIN"
    """str: Save origin status from the specified reading buffer."""
    COL_QUESTIONABLE = "buffer.COL_QUESTIONABLE"
    """str: Save questionable status from the specified reading buffer."""
    COL_RANGE_DIGITS = "buffer.COL_RANGE_DIGITS"
    """str: Save range digits from the specified reading buffer."""
    COL_READING = "buffer.COL_READING"
    """str: Save the measurement reading from the specified reading buffer."""
    COL_STANDARD = "buffer.COL_STANDARD"
    """str: Save the relative time, reading, channel, and source value from the specified reading buffer."""  # noqa: E501
    COL_START = "buffer.COL_START"
    """str: Save the status of the start group from the specified reading buffer."""
    COL_STATUS = "buffer.COL_STATUS"
    """str: Save the status information associated with the measurement from the specified reading buffer."""  # noqa: E501
    COL_TERMINAL = "buffer.COL_TERMINAL"
    """str: Save the terminal status from the specified reading buffer."""
    COL_TIMESTAMP_READING = "buffer.COL_TIMESTAMP_READING"
    """str: Save the timestamp reading from the specified reading buffer."""
    COL_TIME_ABSOLUTE = "buffer.COL_TIME_ABSOLUTE"
    """str: Save the time when the data point was measured as an absolute from the specified reading buffer."""  # noqa: E501
    COL_TIME_PARTS = "buffer.COL_TIME_PARTS"
    """str: Save absolute time in multiple columns from the specified reading buffer."""
    COL_TIME_RAW = "buffer.COL_TIME_RAW"
    """str: Save absolute time in seconds from the specified reading buffer."""
    COL_TIME_RELATIVE = "buffer.COL_TIME_RELATIVE"
    """str: Save the relative time when the data point was measured in seconds from the specified reading buffer."""  # noqa: E501
    COL_UNIT = "buffer.COL_UNIT"
    """str: Save the reading and the unit of measure from the specified reading buffer."""
    DIGITS_3_5 = "buffer.DIGITS_3_5"
    """str: The number of digits to use for the  first measurement."""
    DIGITS_4_5 = "buffer.DIGITS_4_5"
    """str: The number of digits to use for the  first measurement."""
    DIGITS_5_5 = "buffer.DIGITS_5_5"
    """str: The number of digits to use for the  first measurement."""
    DIGITS_6_5 = "buffer.DIGITS_6_5"
    """str: The number of digits to use for the  first measurement."""
    DIGITS_7_5 = "buffer.DIGITS_7_5"
    """str: The number of digits to use for the  first measurement."""
    DIGITS_8_5 = "buffer.DIGITS_8_5"
    """str: The number of digits to use for the  first measurement."""
    EXPR_ADD = "buffer.EXPR_ADD"
    """str: Add the present and previous measurements."""
    EXPR_AVERAGE = "buffer.EXPR_AVERAGE"
    """str: Average the present and previous measurements."""
    EXPR_DIVIDE = "buffer.EXPR_DIVIDE"
    """str: Divide the present reading by the previous reading."""
    EXPR_EXPONENT = "buffer.EXPR_EXPONENT"
    """str: Exponent (10^present reading)."""
    EXPR_LOG10 = "buffer.EXPR_LOG10"
    """str: Log10 (log10(present reading))."""
    EXPR_MULTIPLY = "buffer.EXPR_MULTIPLY"
    """str: Multiply the present and previous measurements."""
    EXPR_NONE = "buffer.EXPR_NONE"
    """str: Remove the math expression."""
    EXPR_POLY = "buffer.EXPR_POLY"
    """str: Polynomial where r is present reading and c is constant."""
    EXPR_POWER = "buffer.EXPR_POWER"
    """str: Present reading raised to power of defined constant."""
    EXPR_RATE = "buffer.EXPR_RATE"
    """str: Rate of change (present reading - previous reading)/(timestamp of present reading - timestamp of previous reading)."""  # noqa: E501
    EXPR_RECIPROCAL = "buffer.EXPR_RECIPROCAL"
    """str: Reciprocal (1/present reading)."""
    EXPR_SQROOT = "buffer.EXPR_SQROOT"
    """str: Square root of the present reading."""
    EXPR_SUBTRACT = "buffer.EXPR_SUBTRACT"
    """str: Present reading - previous reading."""
    FILL_CONTINUOUS = "buffer.FILL_CONTINUOUS"
    """str: Fill the buffer continuously."""
    FILL_ONCE = "buffer.FILL_ONCE"
    """str: Fill the buffer, then stop."""
    OFF = "buffer.OFF"
    """str: Do not log information events."""
    ON = "buffer.ON"
    """str: Log information events."""
    SAVE_FORMAT_TIME = "buffer.SAVE_FORMAT_TIME"
    """str: Include dates, times, and fractional seconds in the buffer."""
    SAVE_RAW_TIME = "buffer.SAVE_RAW_TIME"
    """str: Include seconds and fractional seconds in the buffer."""
    SAVE_RELATIVE_TIME = "buffer.SAVE_RELATIVE_TIME"
    """str: Include relative timestamps in the buffer."""
    SAVE_TIMESTAMP_TIME = "buffer.SAVE_TIMESTAMP_TIME"
    """str: Include timestamps in the buffer."""
    STAT_LIMIT1_HIGH = "buffer.STAT_LIMIT1_HIGH"
    """str: Reading is above high limit 1."""
    STAT_LIMIT1_LOW = "buffer.STAT_LIMIT1_LOW"
    """str: Reading is below low limit 1."""
    STAT_LIMIT2_HIGH = "buffer.STAT_LIMIT2_HIGH"
    """str: Reading is above high limit 2."""
    STAT_LIMIT2_LOW = "buffer.STAT_LIMIT2_LOW"
    """str: Reading is below low limit 2."""
    STAT_ORIGIN = "buffer.STAT_ORIGIN"
    """str: A/D converter from which reading originated; for most instruments, this will always be 0 (main). For digitizing instruments, can be 2 (digitize)."""  # noqa: E501
    STAT_QUESTIONABLE = "buffer.STAT_QUESTIONABLE"
    """str: Measure status questionable."""
    STAT_REL = "buffer.STAT_REL"
    """str: Relative offset."""
    STAT_SCAN = "buffer.STAT_SCAN"
    """str: Scan."""
    STAT_START_GROUP = "buffer.STAT_START_GROUP"
    """str: First reading in a group."""
    STAT_TERMINAL = "buffer.STAT_TERMINAL"
    """str: Measure terminal, front is 1, rear is 0."""
    STYLE_COMPACT = "buffer.STYLE_COMPACT"
    """str: Store readings with reduced accuracy (6.5 digits) with no formatting information, 1 μs accurate timestamp."""  # noqa: E501
    UNIT_AMP = "buffer.UNIT_AMP"
    """str: Set units of measure to dc current."""
    UNIT_AMP_AC = "buffer.UNIT_AMP_AC"
    """str: Set units of measure to ac current."""
    UNIT_CELSIUS = "buffer.UNIT_CELSIUS"
    """str: Set units of measure to Celsius."""
    UNIT_CUSTOM1 = "buffer.UNIT_CUSTOM1"
    """str: Custom unit 1 (defined by buffer.unit())."""
    UNIT_CUSTOM2 = "buffer.UNIT_CUSTOM2"
    """str: Custom unit 2 (defined by buffer.unit())."""
    UNIT_CUSTOM3 = "buffer.UNIT_CUSTOM3"
    """str: Custom unit 3 (defined by buffer.unit())."""
    UNIT_DAC = "buffer.UNIT_DAC"
    """str: Set units of measure to DAC (voltage)."""
    UNIT_DBM = "buffer.UNIT_DBM"
    """str: Set units of measure to decibel-milliwatts."""
    UNIT_DECIBEL = "buffer.UNIT_DECIBEL"
    """str: Set units of measure to decibels."""
    UNIT_DIO = "buffer.UNIT_DIO"
    """str: Set units of measure to digital I/O."""
    UNIT_FAHRENHEIT = "buffer.UNIT_FAHRENHEIT"
    """str: Set units of measure to Fahrenheit."""
    UNIT_FARAD = "buffer.UNIT_FARAD"
    """str: Set units of measure to capacitance."""
    UNIT_HERTZ = "buffer.UNIT_HERTZ"
    """str: Set units of measure to frequency."""
    UNIT_KELVIN = "buffer.UNIT_KELVIN"
    """str: Set units of measure to Kelvin."""
    UNIT_NONE = "buffer.UNIT_NONE"
    """str: Set units of measure to no units."""
    UNIT_OHM = "buffer.UNIT_OHM"
    """str: Set units of measure to ohms."""
    UNIT_PERCENT = "buffer.UNIT_PERCENT"
    """str: Set units of measure to percent."""
    UNIT_RATIO = "buffer.UNIT_RATIO"
    """str: Set units of measure to dc voltage ratio."""
    UNIT_RECIPROCAL = "buffer.UNIT_RECIPROCAL"
    """str: Set units of measure to reciprocal."""
    UNIT_SECOND = "buffer.UNIT_SECOND"
    """str: Set units of measure to period."""
    UNIT_TOT = "buffer.UNIT_TOT"
    """str: Set units of measure to totalizer."""
    UNIT_VOLT = "buffer.UNIT_VOLT"
    """str: Set units of measure to dc voltage."""
    UNIT_VOLT_AC = "buffer.UNIT_VOLT_AC"
    """str: Set units of measure to ac voltage."""
    UNIT_WATT = "buffer.UNIT_WATT"
    """str: Set units of measure to watts."""
    UNIT_X = "buffer.UNIT_X"
    """str: Set units of measure to buffer.UNIT_X."""

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "buffer") -> None:
        super().__init__(device, cmd_syntax)

    def clearstats(self, buffer_var: Optional[str] = None) -> None:
        """Run the ``buffer.clearstats()`` function.

        **Description:**
            - This function clears the statistical information associated with the specified buffer.

        **TSP Syntax:**

        ::

            - buffer.clearstats()

        Args:
            buffer_var (optional): The name of the reading buffer, which may be a default buffer
                (defbuffer1 or defbuffer2) or a user-defined buffer; defaults to defbuffer1 if not
                specified.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (buffer_var,) if x is not None)
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.clearstats({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clearstats()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def delete(self, buffer_name: str) -> None:
        """Run the ``buffer.delete()`` function.

        **Description:**
            - This function deletes a user-defined reading buffer.

        **TSP Syntax:**

        ::

            - buffer.delete()

        Args:
            buffer_name: The name of a user-defined reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.delete({buffer_name})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.delete()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def make(self, buffer_size: str, style: Optional[str] = None) -> str:
        """Run the ``buffer.make()`` function.

        **Description:**
            - This function creates a user-defined reading buffer.

        **TSP Syntax:**

        ::

            - buffer.make()

        Args:
            buffer_size: The maximum number of readings that can be stored in bufferVar; minimum is
                10; 0 to maximize buffer size (see Details).
            style (optional): The type of reading buffer to create.

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    buffer_size,
                    style,
                )
                if x is not None
            )
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.make({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.make()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def save(
        self,
        buffer_var: str,
        file_name: str,
        what: Optional[str] = None,
        start: Optional[str] = None,
        end: Optional[str] = None,
    ) -> None:
        """Run the ``buffer.save()`` function.

        **Description:**
            - This function saves data from the specified reading buffer to a USB flash drive.

        **TSP Syntax:**

        ::

            - buffer.save()

        Args:
            buffer_var: The name of the reading buffer, which may be a default buffer (defbuffer1 or
                defbuffer2) or a user-defined buffer.
            file_name: A string that indicates the name of the file on the USB flash drive in which
                to save the reading buffer.
            what (optional): Defines which information is saved in the file on the USB flash drive.
            start (optional): Defines the starting point in the buffer to start saving data.
            end (optional): Defines the ending point in the buffer to stop saving data.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    buffer_var,
                    f'"{file_name}"',
                    what,
                    start,
                    end,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.save({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.save()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The id commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC,
DSA70KD, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4, MSO4K, MSO4KB, MSO5,
MSO5B, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - ID?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Id(SCPICmdRead):
    """The ``ID`` command.

    **Description:**
        - This query-only command returns identifying information about the instrument and related
          firmware similar to that returned by the ``*IDN?`` IEEE488.2 common query but does not
          include the instrument serial number.

    **Usage:**
        - Using the ``.query()`` method will send the ``ID?`` query.
        - Using the ``.verify(value)`` method will send the ``ID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - ID?
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "ID") -> None:
        super().__init__(device, cmd_syntax)



"""The rosc commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - ROSc:OUT:FREQuency {MHZ10|MHZ100}
    - ROSc:OUT:FREQuency?
    - ROSc:SOUrce {ULTRAsync|INTERnal|EXTernal}{INTERnal|EXTernal|TEKLink}
    - ROSc:SOUrce?
    - ROSc:STATE?
    - ROSc:TRACking {STABle|FAST}
    - ROSc:TRACking?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class RoscTracking(SCPICmdWrite, SCPICmdRead):
    """The ``ROSc:TRACking`` command.

    **Description:**
        - This command sets or queries the tracking mode for the time base reference oscillator. The
          reference oscillator locks to the source. Depending on the command argument that you
          specify, you can use an external reference signal that is fed through or bypasses the
          phase-locked loop. This command is also useful for synchronizing multiple instruments.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc:TRACking?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:TRACking?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ROSc:TRACking value`` command.

    **SCPI Syntax:**

    ::

        - ROSc:TRACking {STABle|FAST}
        - ROSc:TRACking?

    **Info:**
        - ``STABle`` tracking mode specifies that the external reference signal is fed through a
          phase-locked loop that removes jitter from the external reference.
        - ``FAST`` tracking mode specifies that the external reference signal bypasses the
          phase-locked loop.
    """


class RoscState(SCPICmdRead):
    """The ``ROSc:STATE`` command.

    **Description:**
        - This query-only command returns whether the time base reference oscillator is locked. This
          command will return either LOCKED or UNLOCKED.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:STATE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    **SCPI Syntax:**

    ::

        - ROSc:STATE?
    """


class RoscSource(SCPICmdWrite, SCPICmdRead):
    """The ``ROSc:SOUrce`` command.

    **Description:**
        - This command sets or queries the selected source for the time base reference oscillator.
          The reference oscillator locks to this source. Depending on the command argument that you
          specify, you can use an external reference or use the internal crystal oscillator as the
          time base reference. This command is also useful for synchronizing multiple instruments.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ROSc:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - ROSc:SOUrce {ULTRAsync|INTERnal|EXTernal}{INTERnal|EXTernal|TEKLink}
        - ROSc:SOUrce?

    **Info:**
        - ``ULTRAsync`` specifies the 12.5 GHz UltraSync Clock as the time base reference.
        - ``INTERnal`` specifies the internal 10 MHz crystal oscillator as the time base reference.
        - ``EXTernal`` specifies the user-supplied external signal as the time base reference.
        - ``TEKLink`` specifies the TekLink signal as the time base reference.
    """


class RoscOutFrequency(SCPICmdWrite, SCPICmdRead):
    """The ``ROSc:OUT:FREQuency`` command.

    **Description:**
        - This command sets or returns the selected frequency for the timebase reference output
          signal.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc:OUT:FREQuency?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:OUT:FREQuency?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ROSc:OUT:FREQuency value`` command.

    **SCPI Syntax:**

    ::

        - ROSc:OUT:FREQuency {MHZ10|MHZ100}
        - ROSc:OUT:FREQuency?

    **Info:**
        - ``MHZ10`` outputs a 10 MHz reference signal to the REF OUT connector.
        - ``MHZ100`` outputs a 100 MHz reference signal to the REF OUT connector.
    """


class RoscOut(SCPICmdRead):
    """The ``ROSc:OUT`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc:OUT?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc:OUT?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.frequency``: The ``ROSc:OUT:FREQuency`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._frequency = RoscOutFrequency(device, f"{self._cmd_syntax}:FREQuency")

    @property
    def frequency(self) -> RoscOutFrequency:
        """Return the ``ROSc:OUT:FREQuency`` command.

        **Description:**
            - This command sets or returns the selected frequency for the timebase reference output
              signal.

        **Usage:**
            - Using the ``.query()`` method will send the ``ROSc:OUT:FREQuency?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:OUT:FREQuency?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ROSc:OUT:FREQuency value`` command.

        **SCPI Syntax:**

        ::

            - ROSc:OUT:FREQuency {MHZ10|MHZ100}
            - ROSc:OUT:FREQuency?

        **Info:**
            - ``MHZ10`` outputs a 10 MHz reference signal to the REF OUT connector.
            - ``MHZ100`` outputs a 100 MHz reference signal to the REF OUT connector.
        """
        return self._frequency


class Rosc(SCPICmdRead):
    """The ``ROSc`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``ROSc?`` query.
        - Using the ``.verify(value)`` method will send the ``ROSc?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.out``: The ``ROSc:OUT`` command tree.
        - ``.source``: The ``ROSc:SOUrce`` command.
        - ``.state``: The ``ROSc:STATE`` command.
        - ``.tracking``: The ``ROSc:TRACking`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "ROSc") -> None:
        super().__init__(device, cmd_syntax)
        self._out = RoscOut(device, f"{self._cmd_syntax}:OUT")
        self._source = RoscSource(device, f"{self._cmd_syntax}:SOUrce")
        self._state = RoscState(device, f"{self._cmd_syntax}:STATE")
        self._tracking = RoscTracking(device, f"{self._cmd_syntax}:TRACking")

    @property
    def out(self) -> RoscOut:
        """Return the ``ROSc:OUT`` command tree.

        **Usage:**
            - Using the ``.query()`` method will send the ``ROSc:OUT?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:OUT?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.frequency``: The ``ROSc:OUT:FREQuency`` command.
        """
        return self._out

    @property
    def source(self) -> RoscSource:
        """Return the ``ROSc:SOUrce`` command.

        **Description:**
            - This command sets or queries the selected source for the time base reference
              oscillator. The reference oscillator locks to this source. Depending on the command
              argument that you specify, you can use an external reference or use the internal
              crystal oscillator as the time base reference. This command is also useful for
              synchronizing multiple instruments.

        **Usage:**
            - Using the ``.query()`` method will send the ``ROSc:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:SOUrce?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ROSc:SOUrce value`` command.

        **SCPI Syntax:**

        ::

            - ROSc:SOUrce {ULTRAsync|INTERnal|EXTernal}{INTERnal|EXTernal|TEKLink}
            - ROSc:SOUrce?

        **Info:**
            - ``ULTRAsync`` specifies the 12.5 GHz UltraSync Clock as the time base reference.
            - ``INTERnal`` specifies the internal 10 MHz crystal oscillator as the time base
              reference.
            - ``EXTernal`` specifies the user-supplied external signal as the time base reference.
            - ``TEKLink`` specifies the TekLink signal as the time base reference.
        """
        return self._source

    @property
    def state(self) -> RoscState:
        """Return the ``ROSc:STATE`` command.

        **Description:**
            - This query-only command returns whether the time base reference oscillator is locked.
              This command will return either LOCKED or UNLOCKED.

        **Usage:**
            - Using the ``.query()`` method will send the ``ROSc:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:STATE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        **SCPI Syntax:**

        ::

            - ROSc:STATE?
        """
        return self._state

    @property
    def tracking(self) -> RoscTracking:
        """Return the ``ROSc:TRACking`` command.

        **Description:**
            - This command sets or queries the tracking mode for the time base reference oscillator.
              The reference oscillator locks to the source. Depending on the command argument that
              you specify, you can use an external reference signal that is fed through or bypasses
              the phase-locked loop. This command is also useful for synchronizing multiple
              instruments.

        **Usage:**
            - Using the ``.query()`` method will send the ``ROSc:TRACking?`` query.
            - Using the ``.verify(value)`` method will send the ``ROSc:TRACking?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ROSc:TRACking value`` command.

        **SCPI Syntax:**

        ::

            - ROSc:TRACking {STABle|FAST}
            - ROSc:TRACking?

        **Info:**
            - ``STABle`` tracking mode specifies that the external reference signal is fed through a
              phase-locked loop that removes jitter from the external reference.
            - ``FAST`` tracking mode specifies that the external reference signal bypasses the
              phase-locked loop.
        """
        return self._tracking



"""The auxout commands module.

These commands are used in the following models:
DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - AUXOut:SOUrce {ATRIGger|MAIn|REFOut|EVENT|AFG}
    - AUXOut:SOUrce?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class AuxoutSource(SCPICmdWrite, SCPICmdRead):
    """The ``AUXOut:SOUrce`` command.

    **Description:**
        - This command sets (or queries) the source for the Auxiliary Output port.

    **Usage:**
        - Using the ``.query()`` method will send the ``AUXOut:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXOut:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXOut:SOUrce value`` command.

    **SCPI Syntax:**

    ::

        - AUXOut:SOUrce {ATRIGger|MAIn|REFOut|EVENT|AFG}
        - AUXOut:SOUrce?

    **Info:**
        - ``ATRIGger`` is the default signal output of the auxiliary out port. MAIn is synonymous
          with ATRIGger.
        - ``REFOut`` specifies the reference oscillator output as the source for the auxiliary
          output.
        - ``EVENT`` refers to an internally generated oscilloscope event. The Mask and Act on Event
          commands can cause an event output, such as a mask test completion notification event.
        - ``AFG`` specifies the AFG sync out pulse. (Only available for models with the 3-AFG option
          installed.).
    """


class Auxout(SCPICmdRead):
    """The ``AUXOut`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``AUXOut?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXOut?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.source``: The ``AUXOut:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "AUXOut") -> None:
        super().__init__(device, cmd_syntax)
        self._source = AuxoutSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def source(self) -> AuxoutSource:
        """Return the ``AUXOut:SOUrce`` command.

        **Description:**
            - This command sets (or queries) the source for the Auxiliary Output port.

        **Usage:**
            - Using the ``.query()`` method will send the ``AUXOut:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXOut:SOUrce?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXOut:SOUrce value`` command.

        **SCPI Syntax:**

        ::

            - AUXOut:SOUrce {ATRIGger|MAIn|REFOut|EVENT|AFG}
            - AUXOut:SOUrce?

        **Info:**
            - ``ATRIGger`` is the default signal output of the auxiliary out port. MAIn is
              synonymous with ATRIGger.
            - ``REFOut`` specifies the reference oscillator output as the source for the auxiliary
              output.
            - ``EVENT`` refers to an internally generated oscilloscope event. The Mask and Act on
              Event commands can cause an event output, such as a mask test completion notification
              event.
            - ``AFG`` specifies the AFG sync out pulse. (Only available for models with the 3-AFG
              option installed.).
        """
        return self._source



"""The fpanel commands module.

These commands are used in the following models:
DPO5KB, MSO5KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - FPANel:PRESS <menuoff>
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class FpanelPress(SCPICmdWrite):
    """The ``FPANel:PRESS`` command.

    **Description:**
        - This command turns off the displayed menu. It turns off everything except warning
          messages.

    **Usage:**
        - Using the ``.write(value)`` method will send the ``FPANel:PRESS value`` command.

    **SCPI Syntax:**

    ::

        - FPANel:PRESS <menuoff>

    **Info:**
        - ``menuoff`` turns off the menu.
    """


class Fpanel(SCPICmdRead):
    """The ``FPANel`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``FPANel?`` query.
        - Using the ``.verify(value)`` method will send the ``FPANel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.press``: The ``FPANel:PRESS`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "FPANel") -> None:
        super().__init__(device, cmd_syntax)
        self._press = FpanelPress(device, f"{self._cmd_syntax}:PRESS")

    @property
    def press(self) -> FpanelPress:
        """Return the ``FPANel:PRESS`` command.

        **Description:**
            - This command turns off the displayed menu. It turns off everything except warning
              messages.

        **Usage:**
            - Using the ``.write(value)`` method will send the ``FPANel:PRESS value`` command.

        **SCPI Syntax:**

        ::

            - FPANel:PRESS <menuoff>

        **Info:**
            - ``menuoff`` turns off the menu.
        """
        return self._press
